"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("underscore");
exports.MINIMAL = 'MINIMAL';
function dataLevelPlugin(schema, options) {
    if (schema.levelMap == null) {
        schema.levelMap = {};
    }
    schema.levels = [exports.MINIMAL].concat(options.levels || []);
    addToLevelMap(schema, levelPaths(schema));
    schema.pre('find', modifyProjection);
    schema.pre('findOne', modifyProjection);
    schema.pre('findOneAndUpdate', modifyProjection);
}
exports.dataLevelPlugin = dataLevelPlugin;
function modifyProjection(next) {
    if (this.options.level) {
        let schema = this.schema;
        let index = schema.levels.indexOf(this.options.level);
        if (this._fields == null && index >= 0) {
            this._fields = {};
        }
        for (let i = 0; i <= index; i++) {
            for (let field of schema.levelMap[schema.levels[i]]) {
                this._fields[field] = 1;
            }
        }
    }
    next();
}
function addToLevelMap(schema, levelPaths) {
    if (schema.parentSchema) {
        addToLevelMap(schema.parentSchema, levelPaths);
    }
    for (let { path, level } of levelPaths) {
        schema.levelMap[level] = _.union(schema.levelMap[level], [path]);
    }
}
function levelPaths(schema) {
    let res = [];
    schema.eachPath((pathname, schemaType) => {
        res.push({
            path: pathname,
            level: schemaType.options.level || exports.MINIMAL,
        });
    });
    return res;
}

//# sourceMappingURL=data-level.js.map
